@@ -15,7 +15,7 @@ import subprocess
 import sys
 import time
 
-__version__ = "0.0.5"
+__version__ = "0.0.6"
 
 def main():
     parser = argparse.ArgumentParser(description="Synkler")
@@ -36,6 +36,8 @@ def main():
     rsync = config['default']['rsync'] if ('rsync' in config['default']) else None
     rsync_opts = config['default']['rsync_opts'] if ('rsync_opts' in config['default']) else ''
     rsync_opts = list(csv.reader([rsync_opts]))[0]
+    if ('--checksum' not in rsync_opts): rsync_opts.append('--checksum')
+    if ('--partial' not in rsync_opts): rsync_opts.append('--partial')
     synkler_server = config['default']['synkler_server'] if ('synkler_server' in config['default']) else None
 
     if (file_dir is None):
@@ -79,6 +81,7 @@ def main():
     files = {}
     transfer = None
     while (True):
+        # Look at any local files and do what needs doing.
         for f in os.listdir(file_dir):
             if (re.search('^\.', f)):
                 continue
@@ -88,10 +91,10 @@ def main():
             if (f in files):
                 if (size == files[f]['size'] and files[f]['mtime'] == mtime):
                     # The file has stopped changing, we can assume it's no longer being written to -- grab the md5sum.
-                    if (files[f]['md5'] is None):
+                    if (files[f]['md5'] is None or files[f]['state'] == 'upload'):
                         md5 = minorimpact.md5dir(f'{file_dir}/{f}')
                         files[f]['md5'] = md5
-                        files[f]['mod_date'] = int(time.time())
+                        #files[f]['mod_date'] = int(time.time())
                         if (args.debug): minorimpact.fprint(f"{f} md5:{md5}")
                         if (mode == 'upload'):
                             if (args.verbose): minorimpact.fprint(f"new file: {f}")
@@ -99,7 +102,7 @@ def main():
                 else:
                     files[f]['size'] = size
                     files[f]['mtime'] = mtime
-                    files[f]['mod_date'] = int(time.time())
+                    #files[f]['mod_date'] = int(time.time())
             else:
                 if (mode == 'central'):
                     # These files are more than 30 minutes old and haven't been reported in, they can be
@@ -113,6 +116,7 @@ def main():
                 elif (mode == 'upload'):
                     files[f] = {'filename':f, 'pickle_protocol':4, 'mtime':mtime, 'size':size, 'state':'churn', 'md5':None, 'dir':file_dir, 'mod_date':int(time.time()) }
 
+        # Look at any items in the queue and do what needs doing.
         method, properties, body = channel.basic_get( queue=queue_name, auto_ack=True)
         while body != None:
             routing_key = method.routing_key
@@ -126,11 +130,11 @@ def main():
                     if (f not in files):
                         # TODO: Don't just blindly upload everything, set the state to 'new' then verify that we've got space for it
                         #   before setting the state to 'upload'.
-                        if (args.verbose): minorimpact.fprint(f"receiving {f}")
+                        if (args.verbose): minorimpact.fprint(f"{f} ready for upload")
                         files[f] = {'filename':f, 'dir':file_dir, 'size':0, 'mtime':None, 'md5':None, 'state':'upload'}
                     elif (files[f]['size'] == size and files[f]['mtime'] == mtime and files[f]['md5'] == md5):
                         if (files[f]['state'] == 'upload'):
-                            if (args.verbose): minorimpact.fprint(f"supplying {f}")
+                            if (args.verbose): minorimpact.fprint(f"{f} ready for download")
                             files[f]['state'] = 'download'
                     files[f]['mod_date'] = int(time.time())
                 elif (re.match('done', routing_key)):
@@ -141,35 +145,44 @@ def main():
                             if (args.verbose): minorimpact.fprint(f"{f} done")
             elif (mode == 'upload'):
                 if (re.match('upload', routing_key)):
-                    if (args.debug): minorimpact.fprint(f"upload {f}")
+                    if (args.debug): minorimpact.fprint(f"{f} upload requested")
                     if (transfer is None):
                         if (files[f]['state'] == 'new' or (files[f]['state'] == 'uploaded' and int(time.time()) - files[f]['mod_date'] > 60)):
-                            # Start the transfer for new files, or files that we finished transferring more than a minute ago.
                             dest_dir = file_data['dir']
-                            if (dest_dir is not None and (files[f]['md5'] != md5 or files[f]['size'] != size or files[f]['mtime'] != mtime)):
-                                if (files[f]['state'] == 'uploaded' and files[f]['md5'] != md5):
-                                    # It looks like we can sometimes get a bogus md5 when the file is first read, so if central is reporting a different md5,
-                                    #   let's just confirm ours.
+                            if (dest_dir is None):
+                                minorimpact.fprintf("{f} upload failed:  destination directory is not set by central")
+                            else:
+                                # Start the transfer for new files, or files that we finished transferring more than a minute ago.
+                                if (files[f]['state'] == 'uploaded'):
+                                    # It looks like we can sometimes get a bogus md5 when the file is first read, so if we're
+                                    #   re-uploading a file, let's confirm it.
                                     files[f]['md5'] = minorimpact.md5dir(f'{file_dir}/{f}')
+                                    files[f]['state'] = 'new'
+                                    if (args.verbose): minorimpact.fprint(f"{f} upload re-starting")
+                                else:
+                                    if (args.verbose): minorimpact.fprint(f"{f} upload starting")
                                 rsync_command = [rsync, '--archive', '--partial', *rsync_opts, f'{file_dir}/{f}', f'{synkler_server}:{dest_dir}/']
-                                if (args.verbose): minorimpact.fprint(f"sending {f}")
                                 if (args.debug): minorimpact.fprint(' '.join(rsync_command))
                                 transfer = { 'file':f, 'command': rsync_command }
                                 transfer['proc'] = subprocess.Popen(rsync_command)
                                 files[f]['mod_date'] = int(time.time())
+                        else:
+                            if (args.debug): minorimpact.fprint(f"{f} not ready to upload")
                     elif ('file' in transfer and transfer['file'] == f):
                         if (transfer['proc'].poll() is not None):
                             if (transfer['proc'].returncode != 0):
-                                if (args.verbose): minorimpact.fprint(f"failed to send {f}, return code: {return_code}")
+                                minorimpact.fprint(f"{f} upload failed, return code: {return_code}")
                                 files[f]['state'] = 'churn'
                                 files[f]['mod_date'] = int(time.time())
                             else:
                                 files[f]['state'] = 'uploaded'
                                 files[f]['mod_date'] = int(time.time())
-                                if (args.verbose): minorimpact.fprint(f"{f} done")
+                                if (args.verbose): minorimpact.fprint(f"{f} upload completed")
                             transfer = None
                         elif (transfer['proc'].poll() is None):
-                            if (args.debug): minorimpact.fprint(f"still transfering {f}")
+                            if (args.debug): minorimpact.fprint(f"{f} upload in progress")
+                    else:
+                        if (args.debug): minorimpact.fprint(f"waiting on another transfer: {transfer['file']}")
                 elif (re.match('done', routing_key)):
                     if (f in files):
                         files[f]['mod_date'] = int(time.time())
@@ -180,6 +193,8 @@ def main():
                             else:
                                 if (args.verbose): minorimpact.fprint(f"ERROR: {f} on final destination doesn't match, resetting state.")
                                 files[f]['state'] = 'churn'
+                    if (transfer is not None and 'file' in transfer and transfer['file'] == f):
+                        transfer = None
             elif (mode == 'download'):
                  if (re.match('download', routing_key)):
                     file_data = pickle.loads(body)
@@ -189,15 +204,15 @@ def main():
                     size = file_data['size']
                     mtime = file_data['mtime']
 
+                    if (args.debug): minorimpact.fprint(f"{f} download requested")
                     if (f not in files):
-                        if (args.verbose): minorimpact.fprint(f"new file {f}")
                         if (args.debug): minorimpact.fprint(file_data)
                         files[f]  = {'filename':f, 'size':0, 'md5':None, 'mtime':0, 'dir':file_dir, 'state':'download', 'mod_date':int(time.time()) }
 
                     if (files[f]['size'] != size or md5 != files[f]['md5'] or files[f]['mtime'] != mtime):
                         if (transfer is None):
                             rsync_command = [rsync, '--archive', '--partial', *rsync_opts, f'{synkler_server}:"{dir}/{f}"', f'{file_dir}/']
-                            if (args.verbose): minorimpact.fprint(f"downloading {f}")
+                            if (args.verbose): minorimpact.fprint(f"{f} download starting")
                             if (args.debug): minorimpact.fprint(' '.join(rsync_command))
                             transfer = { 'file': f, 'command': rsync_command }
                             transfer['proc'] = subprocess.Popen(rsync_command)
@@ -205,32 +220,41 @@ def main():
                         elif ('file' in transfer and transfer['file'] == f):
                             if (transfer['proc'].poll() is not None):
                                 if (transfer['proc'].returncode != 0):
-                                    if (args.verbose): minorimpact.fprint(f"failed to download {f}, return code: {return_code}")
+                                    if (args.verbose): minorimpact.fprint(f"{f} download failed, return code: {return_code}")
                                     files[f]['mod_date'] = int(time.time())
                                 else:
                                     files[f]['size'] = minorimpact.dirsize(file_dir + '/' + f)
                                     files[f]['mtime'] = os.path.getmtime(file_dir + '/' + f)
                                     files[f]['md5'] = minorimpact.md5dir(file_dir + '/' + f)
                                     files[f]['mod_date'] = int(time.time())
-                                    if (args.verbose): minorimpact.fprint(f"done downloading {f}")
+                                    if (args.verbose): minorimpact.fprint(f"{f} download complete")
                                 transfer = None
                             elif (transfer['proc'].poll() is None):
-                                if (args.debug): minorimpact.fprint(f"still transfering {f}")
+                                if (args.debug): minorimpact.fprint(f"{f} download in progress")
+                        else:
+                            if (args.debug): minorimpact.fprint(f"waiting on another transfer {transfer['file']}")
                     else:
                         if (files[f]['state'] != 'done'):
+                            if (args.verbose): minorimpact.fprint(f"{f} done")
                             files[f]['state'] = 'done'
                             files[f]['mod_date'] = int(time.time())
-                            if (args.verbose): minorimpact.fprint(f"{f} transfer complete")
+                        if (transfer is not None and 'file' in transfer and transfer['file'] == f):
+                            transfer = None
                  elif (re.match('new', routing_key)):
-                     if (f in files):
-                         files[f]['mod_date'] = int(time.time())
-                         if (files[f]['state'] == 'done' and (files[f]['size'] != size or md5 != files[f]['md5'] or files[f]['mtime'] != mtime)):
+                    if (args.debug): minorimpact.fprint(f"{f} available")
+                    if (f in files):
+                        files[f]['mod_date'] = int(time.time())
+                        if (files[f]['state'] == 'done' and (files[f]['size'] != size or md5 != files[f]['md5'] or files[f]['mtime'] != mtime)):
                             del files[f]
                             os.remove(file_dir + '/' + f)
 
             # Get the next item from queue.
             method, properties, body = channel.basic_get( queue=queue_name, auto_ack=True)
 
+        if (transfer is not None and 'file' in transfer and transfer['file'] not in files):
+            transfer = None
+
+        # Go through all the items in our internal status array and do what needs doing.
         filenames = [key for key in files]
         for f in filenames:
             if (mode == 'central'):
@@ -241,14 +265,14 @@ def main():
                     del files[f]
                 elif (int(time.time()) - files[f]['mod_date'] < 30):
                     if (files[f]['state'] == 'upload'):
-                        if (args.debug): print(f"channel upload.{args.id}: {f}")
+                        if (args.debug): minorimpact.fprint(f"write {f} to channel upload.{args.id}")
                         channel.basic_publish(exchange='synkler', routing_key='upload.' + args.id, body=pickle.dumps(files[f], protocol=4))
                     elif (files[f]['state'] == 'download'):
-                        if (args.debug): print(f"channel download.{args.id}: {f}")
+                        if (args.debug): minorimpact.fprint(f"write {f} to channel download.{args.id}")
                         channel.basic_publish(exchange='synkler', routing_key='download.' + args.id, body=pickle.dumps(files[f], protocol=4))
             elif (mode == 'upload'):
                 if (files[f]['state'] in ('new', 'uploaded')):
-                    if (args.debug): print(f"channel new.{args.id}: {f}")
+                    if (args.debug): minorimpact.fprint(f"write {f} to channel new.{args.id}")
                     channel.basic_publish(exchange='synkler', routing_key='new.' + args.id, body=pickle.dumps(files[f]))
                 elif (files[f]['state'] == 'done'):
                     if (cleanup_script is not None):
@@ -261,7 +285,7 @@ def main():
                         if (args.verbose): minorimpact.fprint("running cleanup script:" + ' '.join(command))
                         return_code = subprocess.call(command)
                         if (return_code != 0):
-                            if (args.verbose): minorimpact.fprint(f" ... FAILED: {return_code}")
+                            minorimpact.fprint(f"{f} cleanup script failed: {return_code}")
                         else:
                             if (args.verbose): minorimpact.fprint(" ... done")
                             # Since the file no longer lives in the download directory, delete it from the internal
@@ -274,7 +298,7 @@ def main():
                 if (files[f]['state'] == 'done'):
                     if ((int(time.time()) - files[f]['mod_date']) < 30):
                         # Keep sending the 'done' signal until we haven't heard from the upload server for a full 30 seconds.
-                        if (args.debug): print(f"channel done.{args.id}: {f}")
+                        if (args.debug): minorimpact.fprint(f"write {f} to channel done.{args.id}")
                         channel.basic_publish(exchange='synkler', routing_key='done.' + args.id, body=pickle.dumps(files[f]))
                     else:
                         # Once we're sure the upload server has done its thing, run the cleanup script and delete the file from the array.
@@ -290,7 +314,7 @@ def main():
                             if (return_code == 0):
                                 if (args.verbose): minorimpact.fprint(f" ... done")
                             else:
-                                if (args.verbose): minorimpact.fprint(f" ... FAILED: {return_code}")
+                                minorimpact.fprint(f"{f} cleanup script failed: {return_code}")
                         del files[f]
 
         time.sleep(5)
